@@ -1,6 +1,6 @@
-/* 
+/*
    Mathieu Stefani, 03 mai 2016
-   
+
    This header provides a Result type that can be used to replace exceptions in code
    that has to handle error.
 
@@ -770,25 +770,29 @@ struct Result {
         return !ok_;
     }
 
-    T expect(const char* str) const {
+    T expect(const char* str)
+    {
         if (!isOk()) {
             ::fprintf(stderr, "%s\n", str);
-            std::terminate(); 
+            abort();
         }
         return expect_impl(std::is_same<T, void>());
     }
 
     template<typename Func>
-    auto map(Func func) const {
+    auto map(Func func)
+    {
         using return_type = decltype(func(T{}));
 
         if (this->isOk()) {
-            auto value = this->storage().template get<T>();
-            auto res = func(value);
-            return Result<return_type, E>(types::Ok<return_type>(std::move(res)));
+            auto value = std::move(this->storage().template get<T>());
+            auto res = func(std::move(value));
+            return Result<return_type, E>(
+                types::Ok<return_type>(std::move(res)));
         }
 
-        return Result<return_type, E>(types::Err<E>(this->storage().template get<E>()));
+        return Result<return_type, E>(
+            types::Err<E>(this->storage().template get<E>()));
     }
 
     template<typename Func,
@@ -855,11 +859,11 @@ struct Result {
     template<typename U = T>
     typename std::enable_if<
         !std::is_same<U, void>::value,
-        U
+        T
     >::type
     unwrapOr(const U& defaultValue) const {
         if (isOk()) {
-            return storage().template get<U>();
+            return storage().template get<T>();
         }
         return defaultValue;
     }
@@ -883,7 +887,7 @@ struct Result {
         }
 
         ::fprintf(stderr, "Attempting to unwrap an error Result\n");
-        std::terminate();
+        abort();
     }
 
     template<typename U = T>
@@ -897,35 +901,37 @@ struct Result {
         }
 
         ::fprintf(stderr, "Attempting to unwrap an error Result\n");
-        std::terminate();
+        abort();
     }
 
     template<typename U = T>
-    typename std::enable_if<
-        std::is_same<U, void>::value,
-        U
-    >::type
-    unwrap() const {
+    typename std::enable_if<std::is_same<U, void>::value, U>::type unwrap()
+        const
+    {
         if (isOk()) {
             return;
         }
 
         ::fprintf(stderr, "Attempting to unwrap an error Result\n");
-        std::terminate();
+        abort();
     }
 
-    E unwrapErr() const {
+    E unwrapErr() const
+    {
         if (isErr()) {
             return storage().template get<E>();
         }
 
         ::fprintf(stderr, "Attempting to unwrapErr an ok Result\n");
-        std::terminate();
+        abort();
     }
 
 private:
-    T expect_impl(std::true_type) const { }
-    T expect_impl(std::false_type) const { return storage_.template get<T>(); }
+    T expect_impl(std::true_type) const {}
+    T expect_impl(std::false_type)
+    {
+        return std::move(storage_.template get<T>());
+    }
 
     bool ok_;
     storage_type storage_;
