@@ -21,56 +21,86 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
-
+#include <algorithm>
+#include <iterator>
 #include <regex>
 #include <sstream>
 
-#include "lnav_log.hh"
-#include "is_utf8.hh"
 #include "string_util.hh"
 
-void scrub_to_utf8(char *buffer, size_t length)
+#include "config.h"
+#include "is_utf8.hh"
+#include "lnav_log.hh"
+
+void
+scrub_to_utf8(char* buffer, size_t length)
 {
-    const char *msg;
+    const char* msg;
     int faulty_bytes;
 
     while (true) {
-        ssize_t utf8_end = is_utf8(
-            (unsigned char *) buffer, length, &msg, &faulty_bytes);
+        auto scan_res
+            = is_utf8((unsigned char*) buffer, length, &msg, &faulty_bytes);
 
         if (msg == nullptr) {
             break;
         }
         for (int lpc = 0; lpc < faulty_bytes; lpc++) {
-            buffer[utf8_end + lpc] = '?';
+            buffer[scan_res.usr_end + lpc] = '?';
         }
     }
 }
 
-size_t unquote(char *dst, const char *str, size_t len)
+void
+quote_content(auto_buffer& buf, const string_fragment& sf, char quote_char)
 {
-    if (str[0] == 'r' || str[0] == 'u') {
-        str += 1;
-        len -= 1;
+    for (char ch : sf) {
+        if (ch == quote_char) {
+            buf.push_back('\\').push_back(ch);
+            continue;
+        }
+        switch (ch) {
+            case '\\':
+                buf.push_back('\\').push_back('\\');
+                break;
+            case '\n':
+                buf.push_back('\\').push_back('n');
+                break;
+            case '\t':
+                buf.push_back('\\').push_back('t');
+                break;
+            case '\r':
+                buf.push_back('\\').push_back('r');
+                break;
+            case '\a':
+                buf.push_back('\\').push_back('a');
+                break;
+            case '\b':
+                buf.push_back('\\').push_back('b');
+                break;
+            default:
+                buf.push_back(ch);
+                break;
+        }
     }
-    char quote_char = str[0];
+}
+
+size_t
+unquote_content(char* dst, const char* str, size_t len, char quote_char)
+{
     size_t index = 0;
 
-    require(str[0] == '\'' || str[0] == '"');
-
-    for (size_t lpc = 1; lpc < (len - 1); lpc++, index++) {
+    for (size_t lpc = 0; lpc < len; lpc++, index++) {
         dst[index] = str[lpc];
         if (str[lpc] == quote_char) {
             lpc += 1;
-        }
-        else if (str[lpc] == '\\' && (lpc + 1) < len) {
+        } else if (str[lpc] == '\\' && (lpc + 1) < len) {
             switch (str[lpc + 1]) {
                 case 'n':
                     dst[index] = '\n';
@@ -93,7 +123,22 @@ size_t unquote(char *dst, const char *str, size_t len)
     return index;
 }
 
-size_t unquote_w3c(char *dst, const char *str, size_t len)
+size_t
+unquote(char* dst, const char* str, size_t len)
+{
+    if (str[0] == 'r' || str[0] == 'u') {
+        str += 1;
+        len -= 1;
+    }
+    char quote_char = str[0];
+
+    require(str[0] == '\'' || str[0] == '"');
+
+    return unquote_content(dst, &str[1], len - 2, quote_char);
+}
+
+size_t
+unquote_w3c(char* dst, const char* str, size_t len)
 {
     size_t index = 0;
 
@@ -110,7 +155,8 @@ size_t unquote_w3c(char *dst, const char *str, size_t len)
     return index;
 }
 
-void truncate_to(std::string &str, size_t max_char_len)
+void
+truncate_to(std::string& str, size_t max_char_len)
 {
     static const std::string ELLIPSIS = "\u22ef";
 
@@ -138,23 +184,25 @@ void truncate_to(std::string &str, size_t max_char_len)
     auto chars_to_remove = (str_char_len - max_char_len) + 1;
     auto midpoint = str_char_len / 2;
     auto chars_to_keep_at_front = midpoint - (chars_to_remove / 2);
-    auto bytes_to_keep_at_front =
-        utf8_char_to_byte_index(str, chars_to_keep_at_front);
-    auto remove_up_to_bytes =
-        utf8_char_to_byte_index(str, chars_to_keep_at_front + chars_to_remove);
+    auto bytes_to_keep_at_front
+        = utf8_char_to_byte_index(str, chars_to_keep_at_front);
+    auto remove_up_to_bytes = utf8_char_to_byte_index(
+        str, chars_to_keep_at_front + chars_to_remove);
     auto bytes_to_remove = remove_up_to_bytes - bytes_to_keep_at_front;
     str.erase(bytes_to_keep_at_front, bytes_to_remove);
     str.insert(bytes_to_keep_at_front, ELLIPSIS);
 }
 
-bool is_url(const char *fn)
+bool
+is_url(const std::string& fn)
 {
     static const auto url_re = std::regex("^(file|https?|ftps?|scp|sftp):.*");
 
     return std::regex_match(fn, url_re);
 }
 
-size_t abbreviate_str(char *str, size_t len, size_t max_len)
+size_t
+abbreviate_str(char* str, size_t len, size_t max_len)
 {
     size_t last_start = 1;
 
@@ -183,7 +231,8 @@ size_t abbreviate_str(char *str, size_t len, size_t max_len)
     return len;
 }
 
-void split_ws(const std::string &str, std::vector<std::string> &toks_out)
+void
+split_ws(const std::string& str, std::vector<std::string>& toks_out)
 {
     std::stringstream ss(str);
     std::string buf;
@@ -193,14 +242,16 @@ void split_ws(const std::string &str, std::vector<std::string> &toks_out)
     }
 }
 
-std::string repeat(const std::string& input, size_t num)
+std::string
+repeat(const std::string& input, size_t num)
 {
     std::ostringstream os;
     std::fill_n(std::ostream_iterator<std::string>(os), num, input);
     return os.str();
 }
 
-std::string center_str(const std::string &subject, size_t width)
+std::string
+center_str(const std::string& subject, size_t width)
 {
     std::string retval = subject;
 
@@ -217,33 +268,40 @@ std::string center_str(const std::string &subject, size_t width)
     return retval;
 }
 
-template<typename T>
-size_t strtonum(T &num_out, const char *string, size_t len)
+bool
+is_blank(const std::string& str)
 {
-    size_t retval = 0;
-    T sign = 1;
+    return std::all_of(
+        str.begin(), str.end(), [](const auto ch) { return isspace(ch); });
+}
 
-    num_out = 0;
+std::string
+scrub_ws(const char* in)
+{
+    static const std::string TAB_SYMBOL = "\u21e5";
+    static const std::string LF_SYMBOL = "\u240a";
+    static const std::string CR_SYMBOL = "\u240d";
 
-    for (; retval < len && isspace(string[retval]); retval++);
-    for (; retval < len && string[retval] == '-'; retval++) {
-        sign *= -1;
+    std::string retval;
+
+    for (size_t lpc = 0; in[lpc]; lpc++) {
+        auto ch = in[lpc];
+
+        switch (ch) {
+            case '\t':
+                retval.append(TAB_SYMBOL);
+                break;
+            case '\n':
+                retval.append(LF_SYMBOL);
+                break;
+            case '\r':
+                retval.append(CR_SYMBOL);
+                break;
+            default:
+                retval.append(1, ch);
+                break;
+        }
     }
-    for (; retval < len && string[retval] == '+'; retval++);
-    for (; retval < len && isdigit(string[retval]); retval++) {
-        num_out *= 10;
-        num_out += string[retval] - '0';
-    }
-    num_out *= sign;
 
     return retval;
 }
-
-template
-size_t strtonum<long long>(long long &num_out, const char *string, size_t len);
-
-template
-size_t strtonum<long>(long &num_out, const char *string, size_t len);
-
-template
-size_t strtonum<int>(int &num_out, const char *string, size_t len);
