@@ -21,79 +21,94 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @file column_namer.cc
  */
 
-#include "config.h"
-
 #include <algorithm>
 
-#include "sql_util.hh"
-
-#include "base/lnav_log.hh"
 #include "column_namer.hh"
 
-bool column_namer::existing_name(const std::string &in_name) const
+#include "base/itertools.hh"
+#include "base/lnav_log.hh"
+#include "config.h"
+#include "sql_util.hh"
+
+const char column_namer::BUILTIN_COL[] = "col";
+
+column_namer::column_namer(language lang) : cn_language(lang) {}
+
+bool
+column_namer::existing_name(const string_fragment& in_name) const
 {
-    if (std::binary_search(std::begin(sql_keywords),
-                           std::end(sql_keywords),
-                           toupper(in_name))) {
+    switch (this->cn_language) {
+        case language::SQL: {
+            auto upped = toupper(in_name.to_string());
+
+            if (std::binary_search(
+                    std::begin(sql_keywords), std::end(sql_keywords), upped)) {
+                return true;
+            }
+            break;
+        }
+        case language::JSON:
+            break;
+    }
+
+    if (this->cn_builtin_names | lnav::itertools::find(in_name)) {
         return true;
     }
 
-    if (std::find(this->cn_builtin_names.begin(),
-                  this->cn_builtin_names.end(),
-                  in_name) != this->cn_builtin_names.end()) {
-        return true;
-    }
-
-    if (std::find(this->cn_names.begin(),
-                  this->cn_names.end(),
-                  in_name) != this->cn_names.end()) {
+    if (this->cn_names | lnav::itertools::find(in_name)) {
         return true;
     }
 
     return false;
 }
 
-std::string column_namer::add_column(const std::string &in_name)
+string_fragment
+column_namer::add_column(const string_fragment& in_name)
 {
-    std::string base_name = in_name, retval;
-    size_t      buf_size;
-    int         num = 0;
+    string_fragment base_name;
+    string_fragment retval;
+    fmt::memory_buffer buf;
+    int num = 0;
 
-    buf_size = in_name.length() + 64;
-    char buffer[buf_size];
     if (in_name.empty()) {
-        base_name = "col";
+        base_name = string_fragment::from_const(BUILTIN_COL);
+    } else {
+        base_name = in_name;
     }
 
     retval = base_name;
-
     auto counter_iter = this->cn_name_counters.find(retval);
     if (counter_iter != this->cn_name_counters.end()) {
         num = ++counter_iter->second;
-        snprintf(buffer, buf_size, "%s_%d", base_name.c_str(), num);
-        retval = buffer;
+        fmt::format_to(
+            std::back_inserter(buf), FMT_STRING("{}_{}"), base_name, num);
+        retval = string_fragment::from_memory_buffer(buf);
     }
 
     while (this->existing_name(retval)) {
         if (num == 0) {
-            this->cn_name_counters[retval] = num;
+            auto counter_name = retval.to_owned(this->cn_alloc);
+            this->cn_name_counters[counter_name] = num;
         }
 
-        log_debug("column name already exists: %s", retval.c_str());
-        snprintf(buffer, buf_size, "%s_%d", base_name.c_str(), num);
-        retval = buffer;
+        log_debug(
+            "column name already exists: %.*s", retval.length(), retval.data());
+        fmt::format_to(
+            std::back_inserter(buf), FMT_STRING("{}_{}"), base_name, num);
+        retval = string_fragment::from_memory_buffer(buf);
         num += 1;
     }
 
-    this->cn_names.push_back(retval);
+    retval = retval.to_owned(this->cn_alloc);
+    this->cn_names.emplace_back(retval);
 
     return retval;
 }
