@@ -21,44 +21,42 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
-
 #include <algorithm>
 
-#include "spookyhash/SpookyV2.h"
 #include "data_parser.hh"
 
-using namespace std;
+#include "config.h"
+#include "spookyhash/SpookyV2.h"
 
 data_format data_parser::FORMAT_SEMI("semi", DT_COMMA, DT_SEMI);
 data_format data_parser::FORMAT_COMMA("comma", DT_INVALID, DT_COMMA);
 data_format data_parser::FORMAT_PLAIN("plain", DT_INVALID, DT_INVALID);
 
-data_parser::data_parser(data_scanner *ds)
+data_parser::data_parser(data_scanner* ds)
     : dp_errors("dp_errors", __FILE__, __LINE__),
-      dp_pairs("dp_pairs", __FILE__, __LINE__),
-      dp_msg_format(nullptr),
-      dp_msg_format_begin(ds->get_input().pi_offset),
-      dp_scanner(ds)
+      dp_pairs("dp_pairs", __FILE__, __LINE__), dp_msg_format(nullptr),
+      dp_msg_format_begin(ds->get_init_offset()), dp_scanner(ds)
 {
     if (TRACE_FILE != nullptr) {
-        fprintf(TRACE_FILE, "input %s\n", ds->get_input().get_string());
+        fprintf(TRACE_FILE, "input %s\n", ds->get_input().to_string().c_str());
     }
 }
 
-void data_parser::pairup(data_parser::schema_id_t *schema,
-                         data_parser::element_list_t &pairs_out,
-                         data_parser::element_list_t &in_list, int group_depth)
+void
+data_parser::pairup(data_parser::schema_id_t* schema,
+                    data_parser::element_list_t& pairs_out,
+                    data_parser::element_list_t& in_list,
+                    int group_depth)
 {
     element_list_t ELEMENT_LIST_T(el_stack), ELEMENT_LIST_T(free_row),
-    ELEMENT_LIST_T(key_comps), ELEMENT_LIST_T(value),
-    ELEMENT_LIST_T(prefix);
+        ELEMENT_LIST_T(key_comps), ELEMENT_LIST_T(value),
+        ELEMENT_LIST_T(prefix);
     SpookyHash context;
 
     require(in_list.el_format.df_name != nullptr);
@@ -67,14 +65,12 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
 
     FORMAT_TRACE(in_list);
 
-    for (auto iter = in_list.begin();
-         iter != in_list.end();
-         ++iter) {
+    for (auto iter = in_list.begin(); iter != in_list.end(); ++iter) {
         if (iter->e_token == DNT_GROUP) {
             element_list_t ELEMENT_LIST_T(group_pairs);
 
-            this->pairup(nullptr, group_pairs, *iter->e_sub_elements,
-                         group_depth + 1);
+            this->pairup(
+                nullptr, group_pairs, *iter->e_sub_elements, group_depth + 1);
             if (!group_pairs.empty()) {
                 iter->assign_elements(group_pairs);
             }
@@ -87,16 +83,14 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                 el_stack.PUSH_BACK(*iter);
             }
         } else if (iter->e_token == in_list.el_format.df_terminator) {
-            this->end_of_value(el_stack, key_comps, value, in_list,
-                               group_depth);
+            this->end_of_value(
+                el_stack, key_comps, value, in_list, group_depth);
 
             key_comps.PUSH_BACK(*iter);
         } else if (iter->e_token == in_list.el_format.df_qualifier) {
-            value.SPLICE(value.end(),
-                         key_comps,
-                         key_comps.begin(),
-                         key_comps.end());
-            strip(value, element_if(DT_WHITE));
+            value.SPLICE(
+                value.end(), key_comps, key_comps.begin(), key_comps.end());
+            strip(value, element_is_space{});
             if (!value.empty()) {
                 el_stack.PUSH_BACK(element(value, DNT_VALUE));
             }
@@ -107,8 +101,7 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
             if (!key_comps.empty()) {
                 do {
                     --key_iter;
-                    if (key_iter->e_token ==
-                        in_list.el_format.df_appender) {
+                    if (key_iter->e_token == in_list.el_format.df_appender) {
                         ++key_iter;
                         value.SPLICE(value.end(),
                                      key_comps,
@@ -116,8 +109,9 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                                      key_iter);
                         key_comps.POP_FRONT();
                         found = true;
-                    } else if (key_iter->e_token ==
-                               in_list.el_format.df_terminator) {
+                    } else if (key_iter->e_token
+                               == in_list.el_format.df_terminator)
+                    {
                         std::vector<element> key_copy;
 
                         value.SPLICE(value.end(),
@@ -125,7 +119,7 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                                      key_comps.begin(),
                                      key_iter);
                         key_comps.POP_FRONT();
-                        strip(key_comps, element_if(DT_WHITE));
+                        strip(key_comps, element_is_space{});
                         if (key_comps.empty()) {
                             key_iter = key_comps.end();
                         } else {
@@ -148,33 +142,30 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
             if (!found && !el_stack.empty() && !key_comps.empty()) {
                 element_list_t::iterator value_iter;
 
-                if (el_stack.size() > 1 &&
-                    in_list.el_format.df_appender != DT_INVALID &&
-                    in_list.el_format.df_terminator != DT_INVALID) {
+                if (el_stack.size() > 1
+                    && in_list.el_format.df_appender != DT_INVALID
+                    && in_list.el_format.df_terminator != DT_INVALID)
+                {
                     /* If we're expecting a terminator and haven't found it */
                     /* then this is part of the value. */
                     continue;
                 }
 
-                value.SPLICE(value.end(),
-                             key_comps,
-                             key_comps.begin(),
-                             key_comps.end());
+                value.SPLICE(
+                    value.end(), key_comps, key_comps.begin(), key_comps.end());
                 value_iter = value.end();
                 std::advance(value_iter, -1);
-                key_comps.SPLICE(key_comps.begin(),
-                                 value,
-                                 value_iter,
-                                 value.end());
+                key_comps.SPLICE(
+                    key_comps.begin(), value, value_iter, value.end());
                 key_comps.resize(1);
             }
 
-            strip(value, element_if(DT_WHITE));
+            strip(value, element_is_space{});
             value.remove_if(element_if(DT_COMMA));
             if (!value.empty()) {
                 el_stack.PUSH_BACK(element(value, DNT_VALUE));
             }
-            strip(key_comps, element_if(DT_WHITE));
+            strip(key_comps, element_is_space{});
             if (!key_comps.empty()) {
                 if (key_is_values) {
                     el_stack.PUSH_BACK(element(key_comps, DNT_VALUE));
@@ -198,8 +189,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
     // Only perform the free-row logic at the top level, if we're in a group
     // assume it is a list.
     if (group_depth < 1 && el_stack.empty()) {
-        free_row.SPLICE(free_row.begin(),
-                        key_comps, key_comps.begin(), key_comps.end());
+        free_row.SPLICE(
+            free_row.begin(), key_comps, key_comps.begin(), key_comps.end());
     } else {
         this->end_of_value(el_stack, key_comps, value, in_list, group_depth);
     }
@@ -216,9 +207,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                 element_list_t ELEMENT_LIST_T(free_pair_subs);
                 struct element blank;
 
-                blank.e_capture.c_begin = blank.e_capture.c_end =
-                    el_stack.front().e_capture.
-                        c_begin;
+                blank.e_capture.c_begin = blank.e_capture.c_end
+                    = el_stack.front().e_capture.c_begin;
                 blank.e_token = DNT_KEY;
                 free_pair_subs.PUSH_BACK(blank);
                 free_pair_subs.PUSH_BACK(el_stack.front());
@@ -240,8 +230,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
 
         if (schema != nullptr) {
             size_t key_len;
-            const char *key_val =
-                this->get_element_string(el_stack.front(), key_len);
+            const char* key_val
+                = this->get_element_string(el_stack.front(), key_len);
             context.Update(key_val, key_len);
         }
 
@@ -249,9 +239,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
             element_list_t ELEMENT_LIST_T(free_pair_subs);
             struct element blank;
 
-            blank.e_capture.c_begin = blank.e_capture.c_end =
-                free_row.front().e_capture.
-                    c_begin;
+            blank.e_capture.c_begin = blank.e_capture.c_end
+                = free_row.front().e_capture.c_begin;
             blank.e_token = DNT_KEY;
             free_pair_subs.PUSH_BACK(blank);
             free_pair_subs.PUSH_BACK(free_row.front());
@@ -266,23 +255,23 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
             has_value = true;
         }
 
-        pair_subs.SPLICE(pair_subs.begin(),
-                         el_stack,
-                         el_stack.begin(),
-                         kv_iter);
+        pair_subs.SPLICE(
+            pair_subs.begin(), el_stack, el_stack.begin(), kv_iter);
 
         if (!has_value) {
             element_list_t ELEMENT_LIST_T(blank_value);
-            pcre_input &pi = this->dp_scanner->get_input();
-            const char *str = pi.get_string();
             struct element blank;
 
             blank.e_token = DT_QUOTED_STRING;
-            blank.e_capture.c_begin = blank.e_capture.c_end =
-                pair_subs.front().e_capture.c_end;
-            if ((blank.e_capture.c_begin >= 0) &&
-                ((size_t) blank.e_capture.c_begin < pi.pi_length)) {
-                switch (str[blank.e_capture.c_begin]) {
+            blank.e_capture.c_begin = blank.e_capture.c_end
+                = pair_subs.front().e_capture.c_end;
+            if (blank.e_capture.c_begin >= 0
+                && blank.e_capture.c_begin
+                    < this->dp_scanner->get_input().sf_end)
+            {
+                switch (this->dp_scanner->to_string_fragment(blank.e_capture)
+                            .front())
+                {
                     case '=':
                     case ':':
                         blank.e_capture.c_begin += 1;
@@ -298,12 +287,12 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
     }
 
     if (pairs_out.size() == 1) {
-        element &pair = pairs_out.front();
-        element &evalue = pair.e_sub_elements->back();
+        element& pair = pairs_out.front();
+        element& evalue = pair.e_sub_elements->back();
 
-        if (evalue.e_token == DNT_VALUE &&
-            evalue.e_sub_elements != nullptr &&
-            evalue.e_sub_elements->size() > 1) {
+        if (evalue.e_token == DNT_VALUE && evalue.e_sub_elements != nullptr
+            && evalue.e_sub_elements->size() > 1)
+        {
             element_list_t::iterator next_sub;
 
             next_sub = pair.e_sub_elements->begin();
@@ -343,6 +332,7 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                 case DT_IPV6_ADDRESS:
                 case DT_MAC_ADDRESS:
                 case DT_HEX_DUMP:
+                case DT_XML_DECL_TAG:
                 case DT_XML_OPEN_TAG:
                 case DT_XML_CLOSE_TAG:
                 case DT_XML_EMPTY_TAG:
@@ -355,8 +345,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                     element_list_t ELEMENT_LIST_T(pair_subs);
                     struct element blank;
 
-                    blank.e_capture.c_begin = blank.e_capture.c_end =
-                        free_row.front().e_capture.c_begin;
+                    blank.e_capture.c_begin = blank.e_capture.c_end
+                        = free_row.front().e_capture.c_begin;
                     blank.e_token = DNT_KEY;
                     pair_subs.PUSH_BACK(blank);
                     pair_subs.PUSH_BACK(free_row.front());
@@ -367,17 +357,15 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
                     // use the token ID since some columns values might vary
                     // between rows.
                     context.Update(" ", 1);
-                }
-                    break;
+                } break;
 
                 default: {
                     size_t key_len;
-                    const char *key_val = this->get_element_string(
-                        free_row.front(), key_len);
+                    const char* key_val
+                        = this->get_element_string(free_row.front(), key_len);
 
                     context.Update(key_val, key_len);
-                }
-                    break;
+                } break;
             }
 
             free_row.POP_FRONT();
@@ -388,8 +376,8 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
         element_list_t ELEMENT_LIST_T(pair_subs);
         struct element blank;
 
-        blank.e_capture.c_begin = blank.e_capture.c_end =
-            prefix.front().e_capture.c_begin;
+        blank.e_capture.c_begin = blank.e_capture.c_end
+            = prefix.front().e_capture.c_begin;
         blank.e_token = DNT_KEY;
         pair_subs.PUSH_BACK(blank);
         pair_subs.PUSH_BACK(prefix.front());
@@ -401,23 +389,23 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
     }
 
     if (schema != nullptr && this->dp_msg_format != nullptr) {
-        pcre_input &pi = this->dp_scanner->get_input();
-        for (auto &fiter : pairs_out) {
+        for (auto& fiter : pairs_out) {
             *(this->dp_msg_format) += this->get_string_up_to_value(fiter);
             this->dp_msg_format->append("#");
         }
-        if ((size_t) this->dp_msg_format_begin < pi.pi_length) {
-            const char *str = pi.get_string();
-            pcre_context::capture_t last(this->dp_msg_format_begin,
-                                         pi.pi_length);
+        if ((size_t) this->dp_msg_format_begin
+            < this->dp_scanner->get_input().length())
+        {
+            auto last = this->dp_scanner->get_input().substr(
+                this->dp_msg_format_begin);
 
-            switch (str[last.c_begin]) {
+            switch (last.front()) {
                 case '\'':
                 case '"':
-                    last.c_begin += 1;
+                    last.sf_begin += 1;
                     break;
             }
-            *(this->dp_msg_format) += pi.get_substr(&last);
+            *(this->dp_msg_format) += last.to_string();
         }
     }
 
@@ -426,23 +414,23 @@ void data_parser::pairup(data_parser::schema_id_t *schema,
     }
 }
 
-void data_parser::discover_format()
+void
+data_parser::discover_format()
 {
-    pcre_context_static<30> pc;
     std::stack<discover_format_state> state_stack;
-    struct element elem;
-
     this->dp_group_token.push_back(DT_INVALID);
     this->dp_group_stack.resize(1);
 
     state_stack.push(discover_format_state());
-    while (this->dp_scanner->tokenize2(pc, elem.e_token)) {
-        pcre_context::iterator pc_iter;
+    while (true) {
+        auto tok_res = this->dp_scanner->tokenize2();
+        if (!tok_res) {
+            break;
+        }
 
-        pc_iter = std::find_if(pc.begin(), pc.end(), capture_if_not(-1));
-        require(pc_iter != pc.end());
-
-        elem.e_capture = *pc_iter;
+        element elem;
+        elem.e_token = tok_res->tr_token;
+        elem.e_capture = tok_res->tr_inner_capture;
 
         require(elem.e_capture.c_begin >= 0);
         require(elem.e_capture.c_end >= 0);
@@ -459,7 +447,7 @@ void data_parser::discover_format()
                 break;
 
             case DT_EMPTY_CONTAINER: {
-                auto &curr_group = this->dp_group_stack.back();
+                auto& curr_group = this->dp_group_stack.back();
                 auto empty_list = element_list_t("_anon_", __FILE__, __LINE__);
                 discover_format_state dfs;
 
@@ -468,7 +456,7 @@ void data_parser::discover_format()
                 empty_list.el_format = dfs.dfs_format;
                 curr_group.PUSH_BACK(element());
 
-                auto &empty = curr_group.back();
+                auto& empty = curr_group.back();
                 empty.e_capture.c_begin = elem.e_capture.c_begin + 1;
                 empty.e_capture.c_end = elem.e_capture.c_begin + 1;
                 empty.e_token = DNT_GROUP;
@@ -486,15 +474,16 @@ void data_parser::discover_format()
                     auto riter = this->dp_group_stack.rbegin();
                     ++riter;
                     state_stack.top().finalize();
-                    this->dp_group_stack.back().el_format =
-                        state_stack.top().dfs_format;
+                    this->dp_group_stack.back().el_format
+                        = state_stack.top().dfs_format;
                     state_stack.pop();
                     if (!this->dp_group_stack.back().empty()) {
-                        (*riter).PUSH_BACK(element(this->dp_group_stack.back(),
-                                                   DNT_GROUP));
+                        (*riter).PUSH_BACK(
+                            element(this->dp_group_stack.back(), DNT_GROUP));
                     } else {
                         (*riter).PUSH_BACK(element());
-                        riter->back().e_capture.c_begin = elem.e_capture.c_begin;
+                        riter->back().e_capture.c_begin
+                            = elem.e_capture.c_begin;
                         riter->back().e_capture.c_end = elem.e_capture.c_begin;
                         riter->back().e_token = DNT_GROUP;
                         riter->back().assign_elements(
@@ -519,10 +508,10 @@ void data_parser::discover_format()
         ++riter;
         if (!this->dp_group_stack.back().empty()) {
             state_stack.top().finalize();
-            this->dp_group_stack.back().el_format = state_stack.top().dfs_format;
+            this->dp_group_stack.back().el_format
+                = state_stack.top().dfs_format;
             state_stack.pop();
-            (*riter).PUSH_BACK(element(this->dp_group_stack.back(),
-                                       DNT_GROUP));
+            (*riter).PUSH_BACK(element(this->dp_group_stack.back(), DNT_GROUP));
         }
         this->dp_group_stack.pop_back();
     }
@@ -531,36 +520,39 @@ void data_parser::discover_format()
     this->dp_group_stack.back().el_format = state_stack.top().dfs_format;
 }
 
-void data_parser::end_of_value(data_parser::element_list_t &el_stack,
-                               data_parser::element_list_t &key_comps,
-                               data_parser::element_list_t &value,
-                               const data_parser::element_list_t &in_list,
-                               int group_depth)
+void
+data_parser::end_of_value(data_parser::element_list_t& el_stack,
+                          data_parser::element_list_t& key_comps,
+                          data_parser::element_list_t& value,
+                          const data_parser::element_list_t& in_list,
+                          int group_depth)
 {
     key_comps.remove_if(element_if(in_list.el_format.df_terminator));
     key_comps.remove_if(element_if(DT_COMMA));
     value.remove_if(element_if(in_list.el_format.df_terminator));
     value.remove_if(element_if(DT_COMMA));
-    strip(key_comps, element_if(DT_WHITE));
-    strip(value, element_if(DT_WHITE));
-    if ((el_stack.empty() || el_stack.back().e_token != DNT_KEY) &&
-        value.empty() && key_comps.size() > 1 &&
-        (key_comps.front().e_token == DT_WORD ||
-         key_comps.front().e_token == DT_SYMBOL)) {
+    strip(key_comps, element_is_space{});
+    strip(value, element_is_space{});
+    if ((el_stack.empty() || el_stack.back().e_token != DNT_KEY)
+        && value.empty() && key_comps.size() > 1
+        && (key_comps.front().e_token == DT_WORD
+            || key_comps.front().e_token == DT_SYMBOL))
+    {
         element_list_t::iterator key_iter, key_end;
         bool found_value = false;
         int word_count = 0;
         key_iter = key_comps.begin();
         key_end = key_comps.begin();
         for (; key_iter != key_comps.end(); ++key_iter) {
-            if (key_iter->e_token == DT_WORD ||
-                key_iter->e_token == DT_SYMBOL) {
+            if (key_iter->e_token == DT_WORD || key_iter->e_token == DT_SYMBOL)
+            {
                 word_count += 1;
                 if (found_value) {
                     key_end = key_comps.begin();
                 }
-            } else if (key_iter->e_token == DT_WHITE) {
-
+            } else if (key_iter->e_token == DT_WHITE
+                       || key_iter->e_token == DT_CSI)
+            {
             } else {
                 if (!found_value) {
                     key_end = key_iter;
@@ -571,24 +563,19 @@ void data_parser::end_of_value(data_parser::element_list_t &el_stack,
         if (word_count != 1) {
             key_end = key_comps.begin();
         }
-        value.SPLICE(value.end(),
-                     key_comps,
-                     key_end,
-                     key_comps.end());
-        strip(key_comps, element_if(DT_WHITE));
+        value.SPLICE(value.end(), key_comps, key_end, key_comps.end());
+        strip(key_comps, element_is_space{});
         if (!key_comps.empty()) {
             el_stack.PUSH_BACK(element(key_comps, DNT_KEY, false));
         }
         key_comps.CLEAR();
     } else {
-        value.SPLICE(value.end(),
-                     key_comps,
-                     key_comps.begin(),
-                     key_comps.end());
+        value.SPLICE(
+            value.end(), key_comps, key_comps.begin(), key_comps.end());
     }
-    strip(value, element_if(DT_WHITE));
+    strip(value, element_is_space{});
     strip(value, element_if(DT_COLON));
-    strip(value, element_if(DT_WHITE));
+    strip(value, element_is_space{});
     if (!value.empty()) {
         if (value.size() == 2 && value.back().e_token == DNT_GROUP) {
             element_list_t ELEMENT_LIST_T(group_pair);
@@ -602,34 +589,31 @@ void data_parser::end_of_value(data_parser::element_list_t &el_stack,
     value.CLEAR();
 }
 
-void data_parser::parse()
+void
+data_parser::parse()
 {
     this->discover_format();
 
-    this->pairup(&this->dp_schema_id,
-                 this->dp_pairs,
-                 this->dp_group_stack.front());
+    this->pairup(
+        &this->dp_schema_id, this->dp_pairs, this->dp_group_stack.front());
 }
 
 std::string
-data_parser::get_element_string(const data_parser::element &elem) const
+data_parser::get_element_string(const data_parser::element& elem) const
 {
-    pcre_input &pi = this->dp_scanner->get_input();
-
-    return pi.get_substr(&elem.e_capture);
+    return this->dp_scanner->to_string_fragment(elem.e_capture).to_string();
 }
 
 std::string
-data_parser::get_string_up_to_value(const data_parser::element &elem)
+data_parser::get_string_up_to_value(const data_parser::element& elem)
 {
-    pcre_input &pi = this->dp_scanner->get_input();
-    const element &val_elem = elem.e_token == DNT_PAIR ?
-                              elem.e_sub_elements->back() : elem;
+    const element& val_elem
+        = elem.e_token == DNT_PAIR ? elem.e_sub_elements->back() : elem;
 
     if (this->dp_msg_format_begin <= val_elem.e_capture.c_begin) {
-        pcre_context::capture_t leading_and_key = pcre_context::capture_t(
+        auto leading_and_key = data_scanner::capture_t(
             this->dp_msg_format_begin, val_elem.e_capture.c_begin);
-        const char *str = pi.get_string();
+        auto str = this->dp_scanner->get_input().data();
         if (leading_and_key.length() >= 2) {
             switch (str[leading_and_key.c_end - 1]) {
                 case '\'':
@@ -651,35 +635,37 @@ data_parser::get_string_up_to_value(const data_parser::element &elem)
             }
         }
         this->dp_msg_format_begin = val_elem.e_capture.c_end;
-        return pi.get_substr(&leading_and_key);
+        return this->dp_scanner->to_string_fragment(leading_and_key)
+            .to_string();
     } else {
         this->dp_msg_format_begin = val_elem.e_capture.c_end;
     }
     return "";
 }
 
-const char *data_parser::get_element_string(const data_parser::element &elem,
-                                            size_t &len_out)
+const char*
+data_parser::get_element_string(const data_parser::element& elem,
+                                size_t& len_out)
 {
-    pcre_input &pi = this->dp_scanner->get_input();
-
     len_out = elem.e_capture.length();
-    return pi.get_substr_start(&elem.e_capture);
+    return this->dp_scanner->to_string_fragment(elem.e_capture).data();
 }
 
-void data_parser::print(FILE *out, data_parser::element_list_t &el)
+void
+data_parser::print(FILE* out, data_parser::element_list_t& el)
 {
-    fprintf(out, "             %s\n",
-            this->dp_scanner->get_input().get_string());
-    for (auto &iter : el) {
-        iter.print(out, this->dp_scanner->get_input());
+    fprintf(out,
+            "             %s\n",
+            this->dp_scanner->get_input().to_string().c_str());
+    for (auto& iter : el) {
+        iter.print(out, *this->dp_scanner);
     }
 }
 
-FILE *data_parser::TRACE_FILE;
+FILE* data_parser::TRACE_FILE;
 
-data_format_state_t dfs_prefix_next(data_format_state_t state,
-                                    data_token_t next_token)
+data_format_state_t
+dfs_prefix_next(data_format_state_t state, data_token_t next_token)
 {
     data_format_state_t retval = state;
 
@@ -697,6 +683,7 @@ data_format_state_t dfs_prefix_next(data_format_state_t state,
                 case DT_HEX_NUMBER:
                 case DT_NUMBER:
                 case DT_WHITE:
+                case DT_CSI:
                 case DT_LSQUARE:
                 case DT_RSQUARE:
                 case DT_LANGLE:
@@ -722,8 +709,8 @@ data_format_state_t dfs_prefix_next(data_format_state_t state,
     return retval;
 }
 
-data_format_state_t dfs_semi_next(data_format_state_t state,
-                                  data_token_t next_token)
+data_format_state_t
+dfs_semi_next(data_format_state_t state, data_token_t next_token)
 {
     data_format_state_t retval = state;
 
@@ -777,8 +764,8 @@ data_format_state_t dfs_semi_next(data_format_state_t state,
     return retval;
 }
 
-data_format_state_t dfs_comma_next(data_format_state_t state,
-                                   data_token_t next_token)
+data_format_state_t
+dfs_comma_next(data_format_state_t state, data_token_t next_token)
 {
     data_format_state_t retval = state;
 
@@ -872,25 +859,22 @@ data_format_state_t dfs_comma_next(data_format_state_t state,
 }
 
 data_parser::element::element()
-    : e_capture(-1, -1),
-      e_token(DT_INVALID),
-      e_sub_elements(nullptr)
+    : e_capture(-1, -1), e_token(DT_INVALID), e_sub_elements(nullptr)
 {
 }
 
-data_parser::element::element(data_parser::element_list_t &subs,
-                              data_token_t token, bool assign_subs_elements)
-    : e_capture(subs.front().e_capture.c_begin,
-                subs.back().e_capture.c_end),
-      e_token(token),
-      e_sub_elements(nullptr)
+data_parser::element::element(data_parser::element_list_t& subs,
+                              data_token_t token,
+                              bool assign_subs_elements)
+    : e_capture(subs.front().e_capture.c_begin, subs.back().e_capture.c_end),
+      e_token(token), e_sub_elements(nullptr)
 {
     if (assign_subs_elements) {
         this->assign_elements(subs);
     }
 }
 
-data_parser::element::element(const data_parser::element &other)
+data_parser::element::element(const data_parser::element& other)
 {
     /* require(other.e_sub_elements == nullptr); */
 
@@ -908,8 +892,8 @@ data_parser::element::~element()
     this->e_sub_elements = nullptr;
 }
 
-data_parser::element &
-data_parser::element::operator=(const data_parser::element &other)
+data_parser::element&
+data_parser::element::operator=(const data_parser::element& other)
 {
     this->e_capture = other.e_capture;
     this->e_token = other.e_token;
@@ -920,41 +904,44 @@ data_parser::element::operator=(const data_parser::element &other)
     return *this;
 }
 
-void data_parser::element::assign_elements(data_parser::element_list_t &subs)
+void
+data_parser::element::assign_elements(data_parser::element_list_t& subs)
 {
     if (this->e_sub_elements == nullptr) {
-        this->e_sub_elements = new element_list_t("_sub_", __FILE__,
-                                                  __LINE__);
+        this->e_sub_elements = new element_list_t("_sub_", __FILE__, __LINE__);
         this->e_sub_elements->el_format = subs.el_format;
     }
     this->e_sub_elements->SWAP(subs);
     this->update_capture();
 }
 
-void data_parser::element::update_capture()
+void
+data_parser::element::update_capture()
 {
     if (this->e_sub_elements != nullptr && !this->e_sub_elements->empty()) {
-        this->e_capture.c_begin =
-            this->e_sub_elements->front().e_capture.c_begin;
-        this->e_capture.c_end =
-            this->e_sub_elements->back().e_capture.c_end;
+        this->e_capture.c_begin
+            = this->e_sub_elements->front().e_capture.c_begin;
+        this->e_capture.c_end = this->e_sub_elements->back().e_capture.c_end;
     }
 }
 
-const data_parser::element &data_parser::element::get_pair_value() const
+const data_parser::element&
+data_parser::element::get_pair_value() const
 {
     require(this->e_token == DNT_PAIR);
 
     return this->e_sub_elements->back();
 }
 
-data_token_t data_parser::element::value_token() const
+data_token_t
+data_parser::element::value_token() const
 {
     data_token_t retval = DT_INVALID;
 
     if (this->e_token == DNT_VALUE) {
-        if (this->e_sub_elements != nullptr &&
-            this->e_sub_elements->size() == 1) {
+        if (this->e_sub_elements != nullptr
+            && this->e_sub_elements->size() == 1)
+        {
             retval = this->e_sub_elements->front().e_token;
         } else {
             retval = DT_SYMBOL;
@@ -965,18 +952,21 @@ data_token_t data_parser::element::value_token() const
     return retval;
 }
 
-const data_parser::element &data_parser::element::get_value_elem() const
+const data_parser::element&
+data_parser::element::get_value_elem() const
 {
     if (this->e_token == DNT_VALUE) {
-        if (this->e_sub_elements != nullptr &&
-            this->e_sub_elements->size() == 1) {
+        if (this->e_sub_elements != nullptr
+            && this->e_sub_elements->size() == 1)
+        {
             return this->e_sub_elements->front();
         }
     }
     return *this;
 }
 
-const data_parser::element &data_parser::element::get_pair_elem() const
+const data_parser::element&
+data_parser::element::get_pair_elem() const
 {
     if (this->e_token == DNT_VALUE) {
         return this->e_sub_elements->front();
@@ -984,17 +974,19 @@ const data_parser::element &data_parser::element::get_pair_elem() const
     return *this;
 }
 
-void data_parser::element::print(FILE *out, pcre_input &pi, int offset) const
+void
+data_parser::element::print(FILE* out, data_scanner& ds, int offset) const
 {
     int lpc;
 
     if (this->e_sub_elements != nullptr) {
-        for (auto &e_sub_element : *this->e_sub_elements) {
-            e_sub_element.print(out, pi, offset + 1);
+        for (auto& e_sub_element : *this->e_sub_elements) {
+            e_sub_element.print(out, ds, offset + 1);
         }
     }
 
-    fprintf(out, "%4s %3d:%-3d ",
+    fprintf(out,
+            "%4s %3d:%-3d ",
             data_scanner::token2name(this->e_token),
             this->e_capture.c_begin,
             this->e_capture.c_end);
@@ -1009,27 +1001,27 @@ void data_parser::element::print(FILE *out, pcre_input &pi, int offset) const
             fputc(' ', out);
         }
     }
-    for (; lpc < (int) pi.pi_length; lpc++) {
+    for (; lpc < (int) ds.get_input().length(); lpc++) {
         fputc(' ', out);
     }
 
-    std::string sub = pi.get_substr(&this->e_capture);
+    std::string sub = ds.to_string_fragment(this->e_capture).to_string();
     fprintf(out, "  %s\n", sub.c_str());
 }
 
 data_parser::discover_format_state::discover_format_state()
-    : dfs_prefix_state(DFS_INIT),
-      dfs_semi_state(DFS_INIT),
+    : dfs_prefix_state(DFS_INIT), dfs_semi_state(DFS_INIT),
       dfs_comma_state(DFS_INIT)
 {
     memset(this->dfs_hist, 0, sizeof(this->dfs_hist));
 }
 
-void data_parser::discover_format_state::update_for_element(
-    const data_parser::element &elem)
+void
+data_parser::discover_format_state::update_for_element(
+    const data_parser::element& elem)
 {
-    this->dfs_prefix_state = dfs_prefix_next(this->dfs_prefix_state,
-                                             elem.e_token);
+    this->dfs_prefix_state
+        = dfs_prefix_next(this->dfs_prefix_state, elem.e_token);
     this->dfs_semi_state = dfs_semi_next(this->dfs_semi_state, elem.e_token);
     this->dfs_comma_state = dfs_comma_next(this->dfs_comma_state, elem.e_token);
     if (this->dfs_prefix_state != DFS_ERROR) {
@@ -1043,7 +1035,8 @@ void data_parser::discover_format_state::update_for_element(
     this->dfs_hist[elem.e_token] += 1;
 }
 
-void data_parser::discover_format_state::finalize()
+void
+data_parser::discover_format_state::finalize()
 {
     data_token_t qualifier = this->dfs_format.df_qualifier;
     data_token_t separator = this->dfs_format.df_separator;
@@ -1060,9 +1053,10 @@ void data_parser::discover_format_state::finalize()
     } else if (this->dfs_comma_state != DFS_ERROR) {
         this->dfs_format = FORMAT_COMMA;
         if (separator == DT_COLON && this->dfs_hist[DT_COMMA] > 0) {
-            if (!((this->dfs_hist[DT_COLON] == this->dfs_hist[DT_COMMA]) ||
-                  ((this->dfs_hist[DT_COLON] - 1) ==
-                   this->dfs_hist[DT_COMMA]))) {
+            if (!((this->dfs_hist[DT_COLON] == this->dfs_hist[DT_COMMA])
+                  || ((this->dfs_hist[DT_COLON] - 1)
+                      == this->dfs_hist[DT_COMMA])))
+            {
                 separator = DT_INVALID;
                 if (this->dfs_hist[DT_COLON] == 1) {
                     prefix_term = DT_COLON;
