@@ -21,18 +21,19 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
-
-#include "pcrepp/pcrepp.hh"
 #include "log_level.hh"
 
-const char *level_names[LEVEL__MAX + 1] = {
+#include <ctype.h>
+
+#include "config.h"
+
+const char* level_names[LEVEL__MAX + 1] = {
     "unknown",
     "trace",
     "debug5",
@@ -47,11 +48,13 @@ const char *level_names[LEVEL__MAX + 1] = {
     "error",
     "critical",
     "fatal",
+    "invalid",
 
-    NULL
+    nullptr,
 };
 
-log_level_t abbrev2level(const char *levelstr, ssize_t len)
+log_level_t
+abbrev2level(const char* levelstr, ssize_t len)
 {
     if (len == 0 || levelstr[0] == '\0') {
         return LEVEL_UNKNOWN;
@@ -76,6 +79,13 @@ log_level_t abbrev2level(const char *levelstr, ssize_t len)
             }
             return LEVEL_DEBUG;
         case 'I':
+            if (len == 7 && toupper(levelstr[1]) == 'N'
+                && toupper(levelstr[2]) == 'V' && toupper(levelstr[3]) == 'A'
+                && toupper(levelstr[4]) == 'L' && toupper(levelstr[5]) == 'I'
+                && toupper(levelstr[6]) == 'D')
+            {
+                return LEVEL_INVALID;
+            }
             return LEVEL_INFO;
         case 'S':
             return LEVEL_STATS;
@@ -94,7 +104,8 @@ log_level_t abbrev2level(const char *levelstr, ssize_t len)
     }
 }
 
-int levelcmp(const char *l1, ssize_t l1_len, const char *l2, ssize_t l2_len)
+int
+levelcmp(const char* l1, ssize_t l1_len, const char* l2, ssize_t l2_len)
 {
     return abbrev2level(l1, l1_len) - abbrev2level(l2, l2_len);
 }
