@@ -21,27 +21,41 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
-
-#include "base/string_util.hh"
 #include "pretty_printer.hh"
 
-void pretty_printer::append_to(attr_line_t &al)
+#include "base/string_util.hh"
+#include "config.h"
+
+void
+pretty_printer::append_to(attr_line_t& al)
 {
-    pcre_context_static<30> pc;
-    data_token_t dt;
+    if (this->pp_scanner->get_init_offset() > 0) {
+        data_scanner::capture_t leading_cap = {
+            0,
+            this->pp_scanner->get_init_offset(),
+        };
+
+        // this->pp_stream << pi.get_substr(&leading_cap);
+        this->pp_values.emplace_back(DT_WORD, leading_cap);
+    }
 
     this->pp_scanner->reset();
-    while (this->pp_scanner->tokenize2(pc, dt)) {
-        element el(dt, pc);
+    while (true) {
+        auto tok_res = this->pp_scanner->tokenize2();
+        if (!tok_res) {
+            break;
+        }
 
-        switch (dt) {
+        element el(tok_res->tr_token, tok_res->tr_capture);
+
+        switch (el.e_token) {
+            case DT_XML_DECL_TAG:
             case DT_XML_EMPTY_TAG:
                 if (this->pp_is_xml && this->pp_line_length > 0) {
                     this->start_new_line();
@@ -55,6 +69,10 @@ void pretty_printer::append_to(attr_line_t &al)
                 if (this->pp_is_xml) {
                     this->start_new_line();
                     this->write_element(el);
+                    this->pp_interval_state.back().is_start
+                        = this->pp_stream.tellp();
+                    this->pp_interval_state.back().is_name
+                        = tok_res->to_string();
                     this->descend();
                 } else {
                     this->pp_values.emplace_back(el);
@@ -63,6 +81,7 @@ void pretty_printer::append_to(attr_line_t &al)
             case DT_XML_CLOSE_TAG:
                 this->flush_values();
                 this->ascend();
+                this->append_child_node();
                 this->write_element(el);
                 this->start_new_line();
                 continue;
@@ -72,6 +91,8 @@ void pretty_printer::append_to(attr_line_t &al)
                 this->flush_values(true);
                 this->pp_values.emplace_back(el);
                 this->descend();
+                this->pp_interval_state.back().is_start
+                    = this->pp_stream.tellp();
                 continue;
             case DT_RCURLY:
             case DT_RSQUARE:
@@ -86,14 +107,20 @@ void pretty_printer::append_to(attr_line_t &al)
             case DT_COMMA:
                 if (this->pp_depth > 0) {
                     this->flush_values(true);
+                    if (!this->pp_is_xml) {
+                        this->append_child_node();
+                    }
                     this->write_element(el);
                     this->start_new_line();
+                    this->pp_interval_state.back().is_start
+                        = this->pp_stream.tellp();
                     continue;
                 }
                 break;
             case DT_WHITE:
-                if (this->pp_values.empty() && this->pp_depth == 0 &&
-                    this->pp_line_length == 0) {
+                if (this->pp_values.empty() && this->pp_depth == 0
+                    && this->pp_line_length == 0)
+                {
                     this->pp_leading_indent = el.e_capture.length();
                     continue;
                 }
@@ -116,21 +143,38 @@ void pretty_printer::append_to(attr_line_t &al)
         al.append("\n");
     }
     al.append(combined);
+
+    if (this->pp_hier_stage != nullptr) {
+        this->pp_hier_stage->hn_parent = this->pp_hier_nodes.back().get();
+        this->pp_hier_nodes.back()->hn_children.push_back(
+            std::move(this->pp_hier_stage));
+    }
+    this->pp_hier_stage = std::move(this->pp_hier_nodes.back());
+    this->pp_hier_nodes.pop_back();
+    if (this->pp_hier_stage->hn_children.size() == 1
+        && this->pp_hier_stage->hn_named_children.empty())
+    {
+        this->pp_hier_stage
+            = std::move(this->pp_hier_stage->hn_children.front());
+        this->pp_hier_stage->hn_parent = nullptr;
+    }
 }
 
-void pretty_printer::write_element(const pretty_printer::element &el)
+void
+pretty_printer::write_element(const pretty_printer::element& el)
 {
-    if (this->pp_leading_indent == 0 &&
-        this->pp_line_length == 0 &&
-        el.e_token == DT_WHITE) {
+    if (this->pp_leading_indent == 0 && this->pp_line_length == 0
+        && el.e_token == DT_WHITE)
+    {
         if (this->pp_depth == 0) {
             this->pp_soft_indent += el.e_capture.length();
         }
         return;
     }
-    if (((this->pp_leading_indent == 0) ||
-         (this->pp_line_length <= this->pp_leading_indent)) &&
-        el.e_token == DT_LINE) {
+    if (((this->pp_leading_indent == 0)
+         || (this->pp_line_length <= this->pp_leading_indent))
+        && el.e_token == DT_LINE)
+    {
         this->pp_soft_indent = 0;
         if (this->pp_line_length > 0) {
             this->pp_line_length = 0;
@@ -139,19 +183,20 @@ void pretty_printer::write_element(const pretty_printer::element &el)
         }
         return;
     }
-    pcre_input &pi = this->pp_scanner->get_input();
     if (this->pp_line_length == 0) {
         this->append_indent();
     }
     ssize_t start_size = this->pp_stream.tellp();
     if (el.e_token == DT_QUOTED_STRING) {
-        auto_mem<char> unquoted_str((char *)malloc(el.e_capture.length() + 1));
-        const char *start = pi.get_substr_start(&el.e_capture);
-        unquote(unquoted_str.in(), start, el.e_capture.length());
-        data_scanner ds(unquoted_str.in());
+        auto_mem<char> unquoted_str((char*) malloc(el.e_capture.length() + 1));
+        const char* start
+            = this->pp_scanner->to_string_fragment(el.e_capture).data();
+        auto unq_len = unquote(unquoted_str.in(), start, el.e_capture.length());
+        data_scanner ds(
+            string_fragment::from_bytes(unquoted_str.in(), unq_len));
         string_attrs_t sa;
-        pretty_printer str_pp(&ds, sa,
-                              this->pp_leading_indent + this->pp_depth * 4);
+        pretty_printer str_pp(
+            &ds, sa, this->pp_leading_indent + this->pp_depth * 4);
         attr_line_t result;
         str_pp.append_to(result);
         if (result.get_string().find('\n') != std::string::npos) {
@@ -165,21 +210,20 @@ void pretty_printer::write_element(const pretty_printer::element &el)
                     this->pp_stream << start[0] << start[0];
                     break;
             }
-            this->pp_stream
-                << std::endl
-                << result.get_string();
+            this->pp_stream << std::endl << result.get_string();
             if (result.empty() || result.get_string().back() != '\n') {
                 this->pp_stream << std::endl;
             }
-            this->pp_stream
-                << start[el.e_capture.length() - 1]
-                << start[el.e_capture.length() - 1];
+            this->pp_stream << start[el.e_capture.length() - 1]
+                            << start[el.e_capture.length() - 1];
         } else {
-            this->pp_stream << pi.get_substr(&el.e_capture);
+            this->pp_stream
+                << this->pp_scanner->to_string_fragment(el.e_capture);
         }
     } else {
-        this->pp_stream << pi.get_substr(&el.e_capture);
-        int shift_amount = start_size - el.e_capture.c_begin - this->pp_shift_accum;
+        this->pp_stream << this->pp_scanner->to_string_fragment(el.e_capture);
+        int shift_amount
+            = start_size - el.e_capture.c_begin - this->pp_shift_accum;
         shift_string_attrs(this->pp_attrs, el.e_capture.c_begin, shift_amount);
         this->pp_shift_accum = start_size - el.e_capture.c_begin;
     }
@@ -190,9 +234,11 @@ void pretty_printer::write_element(const pretty_printer::element &el)
     }
 }
 
-void pretty_printer::append_indent()
+void
+pretty_printer::append_indent()
 {
-    this->pp_stream << std::string(this->pp_leading_indent + this->pp_soft_indent, ' ');
+    this->pp_stream << std::string(
+        this->pp_leading_indent + this->pp_soft_indent, ' ');
     this->pp_soft_indent = 0;
     if (this->pp_stream.tellp() == this->pp_leading_indent) {
         return;
@@ -202,17 +248,43 @@ void pretty_printer::append_indent()
     }
 }
 
-bool pretty_printer::flush_values(bool start_on_depth)
+bool
+pretty_printer::flush_values(bool start_on_depth)
 {
+    nonstd::optional<data_scanner::capture_t> last_key;
     bool retval = false;
 
     while (!this->pp_values.empty()) {
         {
-            element &el = this->pp_values.front();
+            auto& el = this->pp_values.front();
             this->write_element(this->pp_values.front());
-            if (start_on_depth &&
-                (el.e_token == DT_LSQUARE ||
-                 el.e_token == DT_LCURLY)) {
+            switch (el.e_token) {
+                case DT_SYMBOL:
+                case DT_CONSTANT:
+                case DT_WORD:
+                case DT_QUOTED_STRING:
+                    last_key = el.e_capture;
+                    break;
+                case DT_COLON:
+                case DT_EQUALS:
+                    if (last_key) {
+                        this->pp_interval_state.back().is_name
+                            = this->pp_scanner
+                                  ->to_string_fragment(last_key.value())
+                                  .to_string();
+                        if (!this->pp_interval_state.back().is_name.empty()) {
+                            this->pp_interval_state.back().is_start
+                                = static_cast<ssize_t>(this->pp_stream.tellp());
+                        }
+                        last_key = nonstd::nullopt;
+                    }
+                    break;
+                default:
+                    break;
+            }
+            if (start_on_depth
+                && (el.e_token == DT_LSQUARE || el.e_token == DT_LCURLY))
+            {
                 if (this->pp_line_length > 0) {
                     this->pp_stream << std::endl;
                 }
@@ -225,7 +297,8 @@ bool pretty_printer::flush_values(bool start_on_depth)
     return retval;
 }
 
-void pretty_printer::start_new_line()
+void
+pretty_printer::start_new_line()
 {
     bool has_output;
 
@@ -241,21 +314,65 @@ void pretty_printer::start_new_line()
     this->pp_body_lines.top() += 1;
 }
 
-void pretty_printer::ascend()
+void
+pretty_printer::ascend()
 {
     if (this->pp_depth > 0) {
         int lines = this->pp_body_lines.top();
         this->pp_depth -= 1;
         this->pp_body_lines.pop();
         this->pp_body_lines.top() += lines;
-    }
-    else {
+
+        if (!this->pp_is_xml) {
+            this->append_child_node();
+        }
+        this->pp_interval_state.pop_back();
+        this->pp_hier_stage = std::move(this->pp_hier_nodes.back());
+        this->pp_hier_nodes.pop_back();
+    } else {
         this->pp_body_lines.top() = 0;
     }
 }
 
-void pretty_printer::descend()
+void
+pretty_printer::descend()
 {
     this->pp_depth += 1;
     this->pp_body_lines.push(0);
+    this->pp_interval_state.resize(this->pp_depth + 1);
+    this->pp_hier_nodes.push_back(
+        std::make_unique<lnav::document::hier_node>());
+}
+
+void
+pretty_printer::append_child_node()
+{
+    auto& ivstate = this->pp_interval_state.back();
+    if (!ivstate.is_start) {
+        return;
+    }
+
+    auto* top_node = this->pp_hier_nodes.back().get();
+    auto new_key = ivstate.is_name.empty()
+        ? lnav::document::section_key_t{top_node->hn_children.size()}
+        : lnav::document::section_key_t{ivstate.is_name};
+    this->pp_intervals.emplace_back(
+        ivstate.is_start.value(),
+        static_cast<ssize_t>(this->pp_stream.tellp()),
+        new_key);
+    auto new_node = this->pp_hier_stage != nullptr
+        ? std::move(this->pp_hier_stage)
+        : std::make_unique<lnav::document::hier_node>();
+    auto* retval = new_node.get();
+    new_node->hn_parent = top_node;
+    new_node->hn_start = this->pp_intervals.back().start;
+    if (!ivstate.is_name.empty()) {
+        top_node->hn_named_children.insert({
+            ivstate.is_name,
+            retval,
+        });
+    }
+    top_node->hn_children.emplace_back(std::move(new_node));
+    ivstate.is_start = nonstd::nullopt;
+    ivstate.is_name.clear();
 }
