@@ -21,8 +21,8 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
@@ -32,20 +32,22 @@
 #include "config.h"
 
 #ifdef HAVE_EXECINFO_H
-#include <execinfo.h>
+#    include <execinfo.h>
 #endif
 
 #include <algorithm>
 
+#include "base/ansi_scrubber.hh"
 #include "shared_buffer.hh"
 
 static const bool DEBUG_TRACE = false;
 
-void shared_buffer_ref::share(shared_buffer &sb, char *data, size_t len)
+void
+shared_buffer_ref::share(shared_buffer& sb, const char* data, size_t len)
 {
 #ifdef HAVE_EXECINFO_H
     if (DEBUG_TRACE) {
-        void *frames[128];
+        void* frames[128];
         int rc;
 
         rc = backtrace(frames, 128);
@@ -60,10 +62,11 @@ void shared_buffer_ref::share(shared_buffer &sb, char *data, size_t len)
     this->sb_data = data;
     this->sb_length = len;
 
-    ensure(this->sb_length < (5 * 1024 * 1024));
+    ensure(this->sb_length < (10 * 1024 * 1024));
 }
 
-bool shared_buffer_ref::subset(shared_buffer_ref &other, off_t offset, size_t len)
+bool
+shared_buffer_ref::subset(shared_buffer_ref& other, off_t offset, size_t len)
 {
     this->disown();
 
@@ -71,11 +74,12 @@ bool shared_buffer_ref::subset(shared_buffer_ref &other, off_t offset, size_t le
         this->sb_owner = other.sb_owner;
         this->sb_length = len;
         if (this->sb_owner == nullptr) {
-            if ((this->sb_data = (char *)malloc(this->sb_length)) == nullptr) {
+            if ((this->sb_data = (char*) malloc(this->sb_length)) == nullptr) {
                 return false;
             }
 
-            memcpy(this->sb_data, &other.sb_data[offset], len);
+            memcpy(
+                const_cast<char*>(this->sb_data), &other.sb_data[offset], len);
         } else {
             this->sb_owner->add_ref(*this);
             this->sb_data = &other.sb_data[offset];
@@ -84,18 +88,20 @@ bool shared_buffer_ref::subset(shared_buffer_ref &other, off_t offset, size_t le
     return true;
 }
 
-shared_buffer_ref::shared_buffer_ref(shared_buffer_ref &&other) noexcept
+shared_buffer_ref::shared_buffer_ref(shared_buffer_ref&& other) noexcept
 {
     if (other.sb_data == nullptr) {
         this->sb_owner = nullptr;
         this->sb_data = nullptr;
         this->sb_length = 0;
     } else if (other.sb_owner != nullptr) {
-        other.sb_owner->add_ref(*this);
-        this->sb_owner = other.sb_owner;
-        this->sb_data = other.sb_data;
-        this->sb_length = other.sb_length;
-        other.disown();
+        auto owner_ref_iter = std::find(other.sb_owner->sb_refs.begin(),
+                                        other.sb_owner->sb_refs.end(),
+                                        &other);
+        *owner_ref_iter = this;
+        this->sb_owner = std::exchange(other.sb_owner, nullptr);
+        this->sb_data = std::exchange(other.sb_data, nullptr);
+        this->sb_length = std::exchange(other.sb_length, 0);
     } else {
         this->sb_owner = nullptr;
         this->sb_data = other.sb_data;
@@ -105,12 +111,13 @@ shared_buffer_ref::shared_buffer_ref(shared_buffer_ref &&other) noexcept
     }
 }
 
-bool shared_buffer_ref::take_ownership()
+bool
+shared_buffer_ref::take_ownership()
 {
     if (this->sb_owner != nullptr && this->sb_data != nullptr) {
-        char *new_data;
+        char* new_data;
 
-        if ((new_data = (char *)malloc(this->sb_length)) == nullptr) {
+        if ((new_data = (char*) malloc(this->sb_length)) == nullptr) {
             return false;
         }
 
@@ -124,11 +131,12 @@ bool shared_buffer_ref::take_ownership()
     return true;
 }
 
-void shared_buffer_ref::disown()
+void
+shared_buffer_ref::disown()
 {
     if (this->sb_owner == nullptr) {
         if (this->sb_data != nullptr) {
-            free(this->sb_data);
+            free(const_cast<char*>(this->sb_data));
         }
     } else {
         this->sb_owner->sb_refs.erase(find(this->sb_owner->sb_refs.begin(),
@@ -140,19 +148,51 @@ void shared_buffer_ref::disown()
     this->sb_length = 0;
 }
 
-void shared_buffer_ref::copy_ref(const shared_buffer_ref &other)
+void
+shared_buffer_ref::copy_ref(const shared_buffer_ref& other)
 {
     if (other.sb_data == nullptr) {
         this->sb_owner = nullptr;
         this->sb_data = nullptr;
         this->sb_length = 0;
-    }
-    else if (other.sb_owner != nullptr) {
+    } else if (other.sb_owner != nullptr) {
         this->share(*other.sb_owner, other.sb_data, other.sb_length);
     } else {
         this->sb_owner = nullptr;
-        this->sb_data = (char *)malloc(other.sb_length);
-        memcpy(this->sb_data, other.sb_data, other.sb_length);
+        this->sb_data = (char*) malloc(other.sb_length);
+        memcpy(
+            const_cast<char*>(this->sb_data), other.sb_data, other.sb_length);
         this->sb_length = other.sb_length;
     }
+    this->sb_metadata = other.sb_metadata;
+}
+
+shared_buffer_ref::narrow_result
+shared_buffer_ref::narrow(size_t new_data, size_t new_length)
+{
+    return std::make_pair(
+        std::exchange(this->sb_data, this->sb_data + new_data),
+        std::exchange(this->sb_length, new_length));
+}
+
+void
+shared_buffer_ref::widen(narrow_result old_data_length)
+{
+    this->sb_data = old_data_length.first;
+    this->sb_length = old_data_length.second;
+}
+
+void
+shared_buffer_ref::erase_ansi()
+{
+    if (!this->sb_metadata.m_has_ansi) {
+        return;
+    }
+
+    auto* writable_data = this->get_writable_data();
+    auto new_len = erase_ansi_escapes(
+        string_fragment::from_bytes(writable_data, this->sb_length));
+
+    this->sb_length = new_len;
+    this->sb_metadata.m_has_ansi = false;
 }
